import os
from pathlib import Path

from IPython import get_ipython
from IPython.core.magic import register_line_magic

from framania.intakemania.extention import FramaniaExtendedIntakeCatalog


@register_line_magic
def start_nb(line):
    parts = line.split(' ')
    if len(parts) != 3:
        print('Make sure input `%start_nb {analysis notebook title} {catalog file path} {data directory path}`')

    analysis_name, catalog_file, data_dir = parts
    
    get_ipython().user_global_ns['__framania_nb_name__'] = analysis_name
    get_ipython().user_global_ns['__framania_catalog_file__'] = Path(catalog_file)
    get_ipython().user_global_ns['__framania_data_dir__'] = Path(data_dir)


@register_line_magic
def begin_analysis(line):
    parts = line.split(' ')
    if len(parts) < 2:
        print(
            'Make sure input `%begin_analysis {analysis name} {analysis version} {upstream analysis name1} {upstream analysis name2} ...`')
    name, version = parts[:2]
    upstream_names = parts[2:]
    get_ipython().user_global_ns['__framania_analysis_name__'] = name
    get_ipython().user_global_ns['__framania_analysis_version__'] = version
    get_ipython().user_global_ns['__framania_analysis_upstream_names__'] = upstream_names

    catalog = FramaniaExtendedIntakeCatalog(get_ipython().user_global_ns['__framania_catalog_file__'])
    upstreams = []
    for upstream_name in upstream_names:
        try:
            upstream = catalog.find_by_version_name(upstream_name)
        except:
            upstream = catalog.find_latest_source_by_name(upstream_name)
        upstreams.append(upstream)
    get_ipython().user_global_ns['__framania_analysis_upstreams__'] = upstreams

    for upstream in upstreams:
        print(f'Register global variable {upstream.name}.')
        get_ipython().user_global_ns[upstream.name] = upstream.intake_source.to_dask()


@register_line_magic
def end_analysis(line):
    if len(line) < 1:
        print('Make sure input `%end_analysis {result dask dataframe variable}`')
    result_var = get_ipython().user_global_ns[line]
    catalog = FramaniaExtendedIntakeCatalog(get_ipython().user_global_ns['__framania_catalog_file__'])
    name = get_ipython().user_global_ns['__framania_analysis_name__']
    version = get_ipython().user_global_ns['__framania_analysis_version__']
    upstreams = get_ipython().user_global_ns['__framania_analysis_upstreams__']
    data_dir = get_ipython().user_global_ns['__framania_data_dir__']
    print(f'Dump dask dataframe {line} to catalog as {name}')
    catalog.dump_dask(result_var, name, version, data_dir, upstreams)

def validation_info_text(info, nest: int, name):
    t = '\t' * nest
    texts = [f'{t}- {name} | {info[name][-1][1]}']

    for upname, upflag in info[name][:-1]:
        texts.append(validation_info_text(info, nest + 1, upname))
    return os.linesep.join(texts)

@register_line_magic
def validate_catalog(line):
    catalog = FramaniaExtendedIntakeCatalog(get_ipython().user_global_ns['__framania_catalog_file__'])
    flag, info = catalog.validate()
    print(f'Validation {flag}')
    for name in sorted(info.keys()):
        print(validation_info_text(info, 0, name))



import hashlib

from dask.dataframe import DataFrame

from framania.pandasmania.util import md5hash as md5hash_pandas


def md5hash(dd: DataFrame) -> hashlib.md5:
    """
    API to make md5hash from pandas dataframe.

    Args:
        pd: pandas dataframe.
    Examples:
        >>> import dask.dataframe
        >>> import pandas
        >>> pd = pandas.DataFrame({'a': [1, 10, 100, 1, 1, 100], 'b': ['a', 'b', 'c', 'd', 'e', 'f'],
        ...                       'idx': [0, 1, 2, 0, 1, 2]}).set_index('idx')
        >>> print(pd)
        ... # doctest: +NORMALIZE_WHITESPACE
               a  b
        idx
        0      1  a
        1     10  b
        2    100  c
        0      1  d
        1      1  e
        2    100  f
        >>> dd = dask.dataframe.from_pandas(pd, npartitions=2)
        >>> md5hash(dd)
        'e650778d6cdca953fad30cccedd4c5f1'
    """
    results = dd.map_partitions(md5hash_pandas).compute()
    m = hashlib.md5()
    for v in results:
        m.update(v.encode('utf-8'))
    return m.hexdigest()




from typing import List, Dict, Union, Any

from dask.dataframe import DataFrame


def aggregate_by_named_index_and_keys(dd: DataFrame, keys: List[Any], agg: Dict[str, Union[str, List[str]]]):
    """
    API for groupby aggregation by named index and other columns.

    Args:
        df (dask.dataframe.DataFrame): target dask dataframe
        keys (List[Any]): groupby key columns that be used with named index
        agg (Dict[str, Union[str, List[str]]]): see `func` in https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.agg.html
    Returns:
        result dask dataframe
    Examples:
        >>> import dask.dataframe
        >>> import pandas
        >>> pd = pandas.DataFrame({'a': [1, 10, 100, 1, 1, 100], 'b': range(0, 600, 100), 'idx': [0, 1, 2, 0, 1, 2]}).set_index('idx')
        >>> print(pd)
        ... # doctest: +NORMALIZE_WHITESPACE
               a    b
        idx
        0      1    0
        1     10  100
        2    100  200
        0      1  300
        1      1  400
        2    100  500
        >>> dd = dask.dataframe.from_pandas(pd, npartitions=2)
        >>> result = aggregate_by_named_index_and_keys(dd, ['a'], {'b': ['mean', 'max']})
        >>> print(result.compute())
        ... # doctest: +NORMALIZE_WHITESPACE
               a    b
                 mean  max
        idx
        0      1  150  300
        1      1  400  400
        1     10  100  100
        2    100  350  500
    """
    index_name = dd.index.name
    groupby_keys = [index_name] + keys

    lambda_agg = lambda pd: pd.groupby(by=groupby_keys).agg(agg)
    lambda_reset_index = lambda pd: pd.reset_index(keys)

    if len(keys) > 0:
        lambda_all = lambda pd: lambda_reset_index(lambda_agg(pd))
    else:
        lambda_all = lambda_agg
    return dd.map_partitions(lambda_all)
